@@ -1,5 +1,5 @@
 
-set rcsId {$Id: demo,v 1.42 1999/03/30 21:19:32 jfontain Exp $}
+set rcsId {$Id: demo,v 1.42.1.1 2000/03/26 09:31:03 jfontain Exp $}
 
 if {[catch {package require stooop 3.7}]} {
 set rcsId {$Id: stooop.tcl,v 3.66 1998/10/28 21:50:15 jfontain Exp $}
@@ -723,8 +723,8 @@ class switched {
 }
 }
 
-if {[catch {package require tkpiechart 5.3}]} {
-set rcsId {$Id: pielabel.tcl,v 1.40 1998/06/07 10:07:30 jfontain Exp $}
+if {[catch {package require tkpiechart 5.4}]} {
+set rcsId {$Id: pielabel.tcl,v 1.40.1.1 2000/03/05 20:54:00 jfontain Exp $}
 
 class pieLabeler {
 
@@ -742,6 +742,8 @@ class pieLabeler {
 
     virtual proc set {this label value}
 
+    virtual proc label {this args}
+
     virtual proc selectState {this label {state {}}}
 
     virtual proc update {this left top right bottom}
@@ -749,7 +751,7 @@ class pieLabeler {
     virtual proc room {this arrayName}
 
 }
-set rcsId {$Id: perilabel.tcl,v 1.47 1999/03/24 21:57:40 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.47.1.1 2000/03/05 21:04:00 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -827,6 +829,14 @@ class piePeripheralLabeler {
         $pieLabeler::($this,canvas) itemconfigure $text -text $value
     }
 
+    proc label {this label args} {
+        if {[llength $args]==0} {
+            return [switched::cget $label -text]
+        } else {
+            switched::configure $label -text [lindex $args 0]
+        }
+    }
+
     proc position {this text slice} {
         variable PI
 
@@ -890,7 +900,7 @@ class piePeripheralLabeler {
     }
 
 }
-set rcsId {$Id: boxlabel.tcl,v 1.41 1999/03/27 21:41:06 jfontain Exp $}
+set rcsId {$Id: boxlabel.tcl,v 1.41.1.1 2000/03/05 20:55:56 jfontain Exp $}
 
 class pieBoxLabeler {
 
@@ -937,10 +947,21 @@ class pieBoxLabeler {
     }
 
     proc set {this label value} {
-        regsub {:.*$} [switched::cget $label -text] ": $value" text
+        regsub {:[^:]*$} [switched::cget $label -text] ": $value" text
         switched::configure $label -text $text
     }
 
+    proc label {this label args} {
+        ::set text [switched::cget $label -text]
+        if {[llength $args]==0} {
+            regexp {^(.*):} $text dummy text
+            return $text
+        } else {
+            regsub {^.*:} $text [lindex $args 0]: text
+            switched::configure $label -text $text
+        }
+    }
+
     proc selectState {this label {selected {}}} {
         if {[string length $selected]==0} {
             return $pieBoxLabeler::($this,selected,$label)
@@ -1440,7 +1461,7 @@ proc slice::data {this arrayName} {
     set data(xCenter) [expr {$x+$data(xRadius)}]
     set data(yCenter) [expr {$y+$data(yRadius)}]
 }
-set rcsId {$Id: selector.tcl,v 1.3 1999/01/31 18:47:07 jfontain Exp $}
+set rcsId {$Id: selector.tcl,v 1.4 1999/08/16 20:59:03 jfontain Exp $}
 
 
 class selector {
@@ -1566,8 +1587,8 @@ class selector {
         variable ${this}selected
 
         ::set list {}
-        foreach index [array names ${this}selected] {
-            if {[::set ${this}selected($index)]} {
+        foreach {index value} [array get ${this}selected] {
+            if {$value} {
                 lappend list $index
             }
         }
@@ -1582,7 +1603,7 @@ class selector {
 
 }
 
-set rcsId {$Id: objselec.tcl,v 1.7 1999/01/31 18:53:01 jfontain Exp $}
+set rcsId {$Id: objselec.tcl,v 1.8 1999/10/02 20:53:05 jfontain Exp $}
 
 
 class objectSelector {
@@ -1609,9 +1630,9 @@ class objectSelector {
     }
 
 }
-set rcsId {$Id: pie.tcl,v 1.85 1999/03/27 21:41:51 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.85.1.2 2000/03/21 21:32:25 jfontain Exp $}
 
-package provide tkpiechart 5.3
+package provide tkpiechart 5.4
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
@@ -1806,6 +1827,10 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     }
 }
 
+proc pie::labelSlice {this slice text} {
+    pieLabeler::label $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $text
+}
+
 proc pie::selectedSlices {this} {
     set list {}
     foreach slice $pie::($this,slices) {
