@@ -1,6 +1,6 @@
-set rcsId {$Id: stooop.tcl,v 3.62 1998/06/05 18:29:09 jfontain Exp $}
+set rcsId {$Id: stooop.tcl,v 3.66 1998/10/28 21:50:15 jfontain Exp $}
 
-package provide stooop 3.6
+package provide stooop 3.7
 
 catch {rename proc _proc}            ;# rename proc before it is overloaded, ignore error in case of multiple inclusion of this file
 
@@ -14,7 +14,7 @@ namespace eval ::stooop {
 
     set checkCode {}                                ;# no checking by default: use an empty instruction to avoid any performance hit
     if {[info exists ::env(STOOOPCHECKALL)]} {
-        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {}}
+        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {} STOOOPCHECKOBJECTS {}}
     }
     if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
         append checkCode {::stooop::checkProcedure;}
@@ -442,24 +442,55 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
                 set interface($fullClass) {}
             }
         }
+    }
 
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        _proc invokingProcedure {} {
+            if {[catch {set procedure [lindex [info level -2] 0]}]} {                                       ;# no invoking procedure
+                return {main script}
+            } elseif {[string length $procedure]==0} {                                              ;# invoked from a namespace body
+                return "namespace [uplevel 2 namespace current]"
+            } else {                                ;# store fully qualified name, visible from creator procedure invoking procedure
+                return [uplevel 3 namespace which -command $procedure]
+            }
+        }
+    }
+
+    if {[info exists ::env(STOOOPCHECKPROCEDURES)]||[info exists ::env(STOOOPCHECKOBJECTS)]} {
         catch {rename ::stooop::new ::stooop::_new}                              ;# gracefully handle multiple sourcing of this file
         _proc ::stooop::new {classOrId args} {           ;# use a new new procedure instead of adding debugging code to existing one
-            variable fullClass
-            variable interface
-
-            if {[scan $classOrId %u dummy]==0} {                                                        ;# first argument is a class
-                set constructor ${classOrId}::[namespace tail $classOrId]                               ;# generate constructor name
-                catch {$constructor}              ;# force loading in case class is in a package so namespace commands work properly
-                set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            } else {                                                                       ;# first argument is an object identifier
-                set fullName $fullClass($classOrId)                         ;# class code, if from a package, must already be loaded
+            variable newId
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                variable fullClass
+                variable interface
             }
-            if {[info exists interface($fullName)]} {
-                error "class $fullName with pure virtual procedures should not be instanciated"
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                variable creator
+            }
+
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                if {[scan $classOrId %u dummy]==0} {                                                    ;# first argument is a class
+                    set constructor ${classOrId}::[namespace tail $classOrId]                           ;# generate constructor name
+                    catch {$constructor}          ;# force loading in case class is in a package so namespace commands work properly
+                    set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
+                    # anticipate full class name storage in original new{} in order to avoid invalid object identifier error in
+                    # checkProcedure{} when member procedure is invoked from within contructor, in which case full class name would
+                    # have yet to be stored.
+                    set fullClass([expr {$newId+1}]) $fullName             ;# new identifier is really incremented in original new{}
+                } else {                                                                   ;# first argument is an object identifier
+                    set fullName $fullClass($classOrId)                     ;# class code, if from a package, must already be loaded
+                }
+                if {[info exists interface($fullName)]} {
+                    error "class $fullName with pure virtual procedures should not be instanciated"
+                }
+            }
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                # keep track of procedure in which creation occured (new identifier is really incremented in original new{})
+                set creator([expr {$newId+1}]) [invokingProcedure]
             }
             return [uplevel ::stooop::_new $classOrId $args]
         }
+
     }
 
     _proc ::stooop::ancestors {fullClass} {                              ;# return the unsorted list of ancestors in class hierarchy
@@ -611,4 +642,49 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
         }
         puts $traceDataChannel $text
     }
+
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        # print existing objects along with creation procedure, with optional class pattern (see the string Tcl command manual)
+        _proc ::stooop::printObjects {{pattern *}} {
+            variable fullClass
+            variable creator
+
+            puts "stooop::printObjects invoked from [invokingProcedure]:"
+            foreach id [lsort -integer [array names fullClass]] {
+                if {[string match $pattern $fullClass($id)]} {
+                    puts "$fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+
+        _proc ::stooop::record {} {                                                  ;# record all existing objects for later report
+            variable fullClass
+            variable checkpointFullClass
+
+            puts "stooop::record invoked from [invokingProcedure]"
+            catch {unset checkpointFullClass}
+            array set checkpointFullClass [array get fullClass]
+        }
+
+        _proc ::stooop::report {{pattern *}} {     ;# print all new or deleted object since last record, with optional class pattern
+            variable fullClass
+            variable checkpointFullClass
+            variable creator
+
+            puts "stooop::report invoked from [invokingProcedure]:"
+            set checkpointIds [lsort -integer [array names checkpointFullClass]]
+            set currentIds [lsort -integer [array names fullClass]]
+            foreach id $currentIds {
+                if {[string match $pattern $fullClass($id)]&&([lsearch -exact $checkpointIds $id]<0)} {
+                    puts "+ $fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+            foreach id $checkpointIds {
+                if {[string match $pattern $checkpointFullClass($id)]&&([lsearch -exact $currentIds $id]<0)} {
+                    puts "- $checkpointFullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+    }
+
 }
